/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_NON_NOISY_PRISM_HH_
# define WIFI_FRAME_FILTER_NON_NOISY_PRISM_HH_

# include <wipal/tool/iterator.hh>
# include <wipal/tool/iterable.hh>

/*---------------------.
| Forward declarations |
`---------------------*/

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

     namespace internals
      {
	template <class, class, class>
	struct non_noisy_prism_iterator;
      }

      template <class, class>
      struct non_noisy_prism;
    }
  }
}


/*--------------.
| Virtual types |
`--------------*/

namespace tool
{

  /**
   * Virtual %types for
   * wifi::frame::filter::internals::non_noisy_prism_iterator.
   *
   * @see types
   */
  template <class I, class B1, class B2>
  struct types< wifi::frame::filter::internals::
		non_noisy_prism_iterator<I, B1, B2> >
  {
    typedef wifi::frame::filter::non_noisy_prism<I, B1>	iterable_type;
    typedef typename I::value_type			value_type;
  };

  /**
   * Virtual %types for wifi::frame::filter::non_noisy_prism.
   *
   * @see types
   */
  template <class I, class B>
  struct types< wifi::frame::filter::non_noisy_prism<I, B> >
  {
    typedef
      wifi::frame::filter::internals::non_noisy_prism_iterator<I, B, bottom>
      iterator;
  };

} // End of namespace tool.


/*-------------.
| Declarations |
`-------------*/

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      namespace internals
      {

	/// Iterator over non-prism-noisy frames.
	template <class I, class B, class Bottom>
	struct non_noisy_prism_iterator:
	  WP_INHERIT(public tool::iterator,
		     non_noisy_prism_iterator<I, B, Bottom>)
	{

	  /// @name Static hierarchies related types
	  //@{
	  typedef WP_GET_EXACT(Bottom, non_noisy_prism_iterator<I, B, Bottom>)
							exact_type;
	  typedef tool::iterator<exact_type>		super_type;
	  typedef WP_TYPE(value_type, exact_type)	value_type;
	  typedef WP_TYPE(iterable_type, exact_type)	iterable_type;
	  //@}

	  /// tool::iterable compatible constructor.
	  non_noisy_prism_iterator(const iterable_type& i, bool end);

	  /// @name Abstract method implementations
	  //@{
	  bool		equal(const non_noisy_prism_iterator& rhs) const;
	  void		increment();

	  const value_type&	get() const;
	  value_type&		get();

	  const value_type*	get_ptr() const;
	  value_type*		get_ptr();
	  //@}

	private:
	  /// Return true when a frame has a non-zero Prism noise.
	  static bool		noisy(const void* frame, size_t caplen);

	  /// Skip noisy frames, starting from @c current_.
	  void			skip_noisy();

	  const iterable_type*	iterable_;
	  I			current_;
	};

      } // End of namespace wifi::frame::filter::internals.


      /// Filter out PCAP frames which Prism noise field is non-null.
      template <class InputIterator, class Bottom = tool::bottom>
      struct non_noisy_prism: WP_INHERIT(public tool::iterable,
					 non_noisy_prism<InputIterator, Bottom>)
      {
	/// Construct from begin/end iterators.
	non_noisy_prism(const InputIterator& begin, const InputIterator& end);

	/// Copy from another iterable object.
	template <class Iterable>
	non_noisy_prism(const Iterable& i);

      private:
	InputIterator		begin_;
	InputIterator		end_;

	friend class internals::non_noisy_prism_iterator<InputIterator,
							 Bottom, tool::bottom>;
      };

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "non_noisy_prism.hxx"

#endif // ! WIFI_FRAME_FILTER_NON_NOISY_PRISM_HH_
