/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_EPOCH_TRACKER_HH_
# define WIFI_FRAME_STATS_EPOCH_TRACKER_HH_

# include <vector>
# include <stdexcept>

# include <boost/optional.hpp>

# include <wipal/tool/microseconds.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      template <class T, unsigned EpochLength>
      struct epoch_tracker
      {
	epoch_tracker();

	void			restart();
	bool			update(const tool::microseconds& timestamp);
	const std::vector<T>&	epochs() const;
	std::vector<T>&		epochs();
	const T&		current() const;
	T&			current();

      private:
	std::vector<T>				epochs_;
	boost::optional<tool::microseconds>	epoch_start_;
	unsigned				epoch_idx_;
      };

      struct non_increasing_timestamp: public std::invalid_argument
      {
	non_increasing_timestamp(const std::string& what);
      };

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "epoch_tracker.hxx"

#endif // ! WIFI_FRAME_STATS_EPOCH_TRACKER_HH_
