/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_GAP_LENGTHS_HXX_
# define WIFI_FRAME_STATS_GAP_LENGTHS_HXX_

# include <cassert>

# include <wipal/wifi/frame/stats/gap_lengths.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      inline
      gap_lengths::gap_lengths(const void*): gl_freqs_ (gl_freqs_size),
					     gap_count_ (0)
      {
      }

      inline
      void
      gap_lengths::account_gap(const unsigned& gap_length)
      {
	assert(0 < gap_length);
	assert(gap_length < gl_max);

	++gl_freqs_[gap_length / gl_freqs_resolution];
	++gap_count_;
      }

      inline
      std::ostream&
      gap_lengths::print(std::ostream& o) const
      {
	o << "begin gap length frequencies\n";

	for (unsigned i = 0; i < gl_freqs_.size(); ++i)
	  {
	    const unsigned	f = gl_freqs_[i];

	    if (not f)
	      continue;

	    o << "  " << (i * gl_freqs_resolution + (gl_freqs_resolution / 2))
	      << '\t' << (double (f) / gap_count_) << '\n';
	  }
	return o << "end gap length frequencies" << std::endl;
      }


    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_STATS_GAP_LENGTHS_HXX_
