/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TRACES_MERGE_HXX_
# define WIFI_TRACES_MERGE_HXX_

# include "merge.hh"

# include <stdexcept>

# include <wipal/pcap/dumper.hh>
# include <wipal/pcap/list_of_traces.hh>
# include <wipal/wifi/frame/filter/merge.hh>

namespace wifi
{

  namespace traces
  {

    /// Internal helpers for merge().
    namespace merge_helpers
    {

      struct dump
      {
	dump(pcapxx::dumper& d): d_ (d)
	{
	}

	template <class Merger>
	void
	operator () (const Merger& m)
	{
	  m.for_each(d_);
	}

      private:
	pcapxx::dumper&	d_;
      };

      template <class Trace>
      pcapxx::dumper
      pcap_dumper(const Trace& t, const std::string& output)
      {
	return t.dumper(output);
      }

      template <>
      pcapxx::dumper
      pcap_dumper(const pcapxx::list_of_traces<>& t, const std::string& output)
      {
	if (t.empty())
	  throw std::invalid_argument ("Empty trace");

	return t.get(0).dumper(output);
      }

    } // End of namespace wifi::traces::merge_helpers.

    template <class UniqueId, class PhyHeader,
	      template <class, class, class> class Intersector,
	      template <class, class, class, class> class Merger,
	      class T1, class T2>
    void
    merge(T1&			t1,
	  T2&			t2,
	  const std::string&	output,
	  tool::opt::list&	options)
    {
      pcapxx::dumper		pcap_dump = (merge_helpers::
					     pcap_dumper(t2, output));
      merge_helpers::dump	dump (pcap_dump);


      frame::filter::provide_merge<UniqueId, PhyHeader, Intersector, Merger>
	(t1.begin(), t1.end(), t2.begin(), t2.end(), options, dump);
    }

  } // End of namespace wifi::traces.

} // End of namespace wifi.

#endif // ! WIFI_TRACES_MERGE_HXX_
