/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <stdexcept>
#include <iostream>
#include <cstdlib>

#include <wipal/tool/args.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/pcap/dumper.hh>
#include <wipal/wifi/frame/filter/non_noisy_prism.hh>

int
main(int argc, char *const argv[])
{
  try
    {
      namespace opt = tool::opt;
      namespace args = tool::args;
      namespace def = tool::args::def;

      opt::list opts ((opt::name ("internals") -= opt::list
		       (( opt::name ("ignore_noisy_prism") -=
						 def::ignore_noisy_prism()))));

      args::parse(argc, argv, opts, args::count (1), args::count (1));

      typedef pcapxx::list_of_traces<>	list_of_traces;

      const args::input&	input = opt::get<args::input>(opts["proc"]
							      ["input"]);
      list_of_traces		traces (input[0]);

      if (traces.empty())
	throw std::invalid_argument ("Empty trace");

      const args::output&	output = opt::get<args::output>(opts["proc"]
								["output"]);
      pcapxx::dumper		dump = traces.get(0).dumper(output[0]);

      if (opt::get<bool>(opts["internals"]["ignore_noisy_prism"]))
	{
	  typedef list_of_traces::const_iterator		li;
	  typedef wifi::frame::filter::non_noisy_prism<li>	nnp;

	  traces.expect(pcapxx::descriptor<>::PRISM_HEADER);

	  nnp			filter (traces.begin(), traces.end());

	  filter.for_each(dump);
	}
      else
	{
	  traces.for_each(dump);
	}
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
