#include <map>
#include <stdexcept>
#include <cstdlib>

#include <boost/filesystem.hpp>

#include <wipal/tool/mapping.hh>

namespace
{
  static std::map<int, int>	loaded;

  struct test_policy
  {
    typedef std::map<int, int>	map;

    struct nv: public std::exception
    {
      nv(int x): x (x)
      {
      }

      int x;
    };

    int
    new_value(int x, const map&)
    {
      throw nv (x);
    }

    void
    value_loaded(int x, int y, const map&)
    {
      loaded[x] = y;
    }
  };

} // End of anonymous namespace.

int
main()
{
  typedef tool::mapping<int, int, test_policy>	mapping;

  const char*		srcdir_p  = getenv("srcdir");

  if (not srcdir_p)
    abort();

  const boost::filesystem::path	srcdir (srcdir_p);
  mapping			valid ((srcdir / "valid.map").file_string());

  if (loaded.size() != 1 or not loaded.count(42) or loaded[42] != 42)
    abort();
  if (valid[42] != 42)
    abort();
  try
    {
      const int x = valid[51];
    }
  catch (const test_policy::nv& nv)
    {
      if (nv.x != 51)
	abort();
    }

  bool	error = true;
  try
    {
      mapping		invalid0 ((srcdir / "invalid0.map").file_string());
    }
  catch (const std::invalid_argument&)
    {
      error = false;
    }
  if (error)
    abort();
  error = true;
  try
    {
      mapping		invalid1 ((srcdir / "invalid1.map").file_string());
    }
  catch (const std::invalid_argument&)
    {
      error = false;
    }
  if (error)
    abort();
}
